@@ -1,9 +1,9 @@
 
-__version__ = "7.10"
+__version__ = "7.20"
 __author__ = "Alex Al-Saffar"
 __date__ = "2022-08-01"
 docs = r"https://crocodile.readthedocs.io/en/latest/"
 git = r"https://github.com/thisismygitrepo/crocodile"
 release_notes = """
-LTS Stable Release.
+Fixed bugs in SSH
 """
