@@ -56,7 +56,7 @@ class DBMS:
     def from_local_db(cls, path=None, echo=False): return cls(engine=cls.make_sql_engine(path=path, echo=echo))
     def __repr__(self): return f"DataBase @ {self.eng}"
     def get_columns(self, table, sch=None): return self.meta.tables[self._get_table_identifier(table, sch)].exported_columns.keys()
-    def close(self, sleep=2): self.con.close(); self.ses.close(); self.eng.dispose(); time.sleep(sleep)
+    def close(self, sleep=2): print(f"Terminating database `{self.path.as_uri()}`"); self.con.close(); self.ses.close(); self.eng.dispose(); time.sleep(sleep)
     def _get_table_identifier(self, table, sch):
         if sch is None: sch = self.sch
         if sch is not None: return sch + "." + table
@@ -66,7 +66,7 @@ class DBMS:
     def make_sql_engine(path=None, echo=False, dialect="sqlite", driver=["pysqlite", "DBAPI"][0]):
         """Establish lazy initialization with database"""
         if path == "memory": return create_engine(url=f"{dialect}+{driver}:///:memory:", echo=echo, future=True)
-        path = tb.P.tmpfile(folder="tmp_dbs", suffix=".db") if path is None else tb.P(path).expanduser().absolute()
+        path = tb.P.tmpfile(folder="tmp_dbs", suffix=".db") if path is None else tb.P(path).expanduser().absolute().create(parents_only=True)
         print(f"Linking to database at {path.as_uri()}")
         return create_engine(url=f"{dialect}+{driver}:///{path}", echo=echo, future=True) # echo flag is just a short for the more formal way of logging sql commands.
 
